@@ -20,7 +20,9 @@ import hashlib
 import json
 import shutil
 import subprocess
+import tempfile
 import time
+import traceback
 from pathlib import Path
 from subprocess import CompletedProcess
 from threading import Thread
@@ -28,22 +30,23 @@ from typing import Dict, Optional
 
 import click
 import paho.mqtt.client as mqtt
+import pkg_resources
 
 
 class NLUTrainer(object):
 
 	TOPIC_READY             = 'projectalice/nlu/trainerReady'
 	TOPIC_STOPPED           = 'projectalice/nlu/trainerStopped'
-	TOPIC_TRAINING          = 'projectalice/nlu/training'
-	TOPIC_TRAINING_STATUS   = 'projectalice/nlu/trainingStatus'
 
-	TOPIC_TRAIN             = 'projectalice/nlu/doTrain'
+	TOPIC_TRAINING          = 'projectalice/nlu/training'
 	TOPIC_REFUSE_FAILED     = 'projectalice/nlu/trainingFailed'
 	TOPIC_TRAINING_RESULT   = 'projectalice/nlu/trainingResult/{}'
 
+	TOPIC_TRAIN             = 'projectalice/nlu/doTrain'
 	TOPIC_CORE_RECONNECTION = 'projectalice/devices/coreReconnection'
+	TOPIC_TRAINING_STATUS   = 'projectalice/nlu/trainingStatus'
 
-	DATASET_FILE = Path('snipsNluDataset.json')
+	DATASET_FILE = Path(tempfile.NamedTemporaryFile().name)
 	DEBUG_DATA_FILE = Path('debugDataset.json')
 
 	def __init__(self, hostname: str = 'localhost', port: int = 1883, user: str = '', password: str = '', tlsFile: str = ''):
@@ -88,6 +91,7 @@ class NLUTrainer(object):
 	def onMqttMessage(self, _client, _userdata, message: mqtt.MQTTMessage):
 		if message.topic == self.TOPIC_CORE_RECONNECTION:
 			self._mqttClient.publish(topic=self.TOPIC_READY)
+			print('Alice main unit just connected')
 		elif message == self.TOPIC_TRAINING_STATUS:
 			self._mqttClient.publish(self.TOPIC_TRAINING_STATUS, payload=json.dumps({'status': 'training' if self._training else 'done'}))
 		elif message.topic == self.TOPIC_TRAIN:
@@ -98,7 +102,7 @@ class NLUTrainer(object):
 					raise Exception('No payload in message')
 
 				payload = json.loads(message.payload.decode())
-				data = json.loads(payload.get('data', '{}'))
+				data = payload.get('data', dict())
 				language = payload.get('language', None)
 
 				if not data:
@@ -119,6 +123,7 @@ class NLUTrainer(object):
 
 
 	def failedTraining(self, reason: str):
+		traceback.print_exc()
 		self._mqttClient.publish(
 			topic=self.TOPIC_REFUSE_FAILED,
 			payload=reason
@@ -126,7 +131,7 @@ class NLUTrainer(object):
 
 
 	def train(self, language: str, trainingData: Dict):
-		print('Received training request')
+		print('Preparing dataset')
 
 		if self._training:
 			reason = "Already training, can't train now"
@@ -163,33 +168,28 @@ class NLUTrainer(object):
 			self._mqttClient.publish(topic=self.TOPIC_TRAINING)
 
 			print(f'Download language support for {language}')
-			download: CompletedProcess = subprocess.run([f'snips-nlu', 'download', language], shell=True, check=True)
+			download: CompletedProcess = subprocess.run(f'snips-nlu download {language}', shell=True, check=True)
 			if download.returncode != 0 :
 				raise Exception(download.stderr.decode())
 
-			print('Begin training...')
+			print('Begin training')
 
-			trainedNLU = Path('trainedNLU')
-			if trainedNLU.exists():
-				shutil.rmtree(trainedNLU, ignore_errors=True)
-
-			training: CompletedProcess = subprocess.run([f'snips-nlu', 'train', str(self.DATASET_FILE), str(trainedNLU)], shell=True, check=True)
+			trainedNLU = Path(tempfile.TemporaryDirectory().name)
+			training: CompletedProcess = subprocess.run(f'snips-nlu train {str(self.DATASET_FILE)} {str(trainedNLU)}', shell=True, check=True)
 			if training.returncode != 0 or not trainedNLU.exists():
 				raise Exception(training.stderr.decode())
 
-			trainedNLU = Path('trainedNLU.zip')
-			if trainedNLU.exists():
-				trainedNLU.unlink()
-
-			shutil.make_archive('trainedNLU', 'zip', 'trainedNLU')
+			archive = Path(tempfile.TemporaryDirectory().name, 'archive')
+			data = Path(shutil.make_archive(str(archive), 'zip', str(trainedNLU))).read_bytes()
+			timer = round(time.time() - startTime, ndigits=2)
+			topic = self.TOPIC_TRAINING_RESULT.format(hashlib.blake2b(archive.with_suffix('.zip').read_bytes()).hexdigest())
 
 			print(f'Sending results')
 			self._mqttClient.publish(
-				topic=self.TOPIC_TRAINING_RESULT.format(hashlib.blake2b(trainedNLU.read_bytes()).hexdigest()),
-				payload=trainedNLU.read_bytes(),
+				topic=topic,
+				payload=data,
 				qos=0
 			)
-			timer = round(time.time() - startTime, ndigits=2)
 			print(f'Training done! It took {timer} seconds to train.')
 		except Exception as e:
 			reason = f'Training failed: {e}'
@@ -202,6 +202,8 @@ class NLUTrainer(object):
 	def onConnect(self, _client, _userdata, _flags, _rc):
 		print('Mqtt connected, listening for training tasks...')
 		self._mqttClient.subscribe(self.TOPIC_TRAIN)
+		self._mqttClient.subscribe(self.TOPIC_TRAINING_STATUS)
+		self._mqttClient.subscribe(self.TOPIC_CORE_RECONNECTION)
 		self._mqttClient.publish(topic=self.TOPIC_READY)
 
 
@@ -218,7 +220,8 @@ class NLUTrainer(object):
 @click.option('-s', '--password', default='', help='Mqtt server password if required')
 @click.option('-t', '--tls_file', default='', help='Path to TLS certificate file, if required')
 def start(host: str, port: int = 1883, user: str = '', password: str = '', tls_file: str = ''): #NOSONAR
-	print('Starting Project Alice decentralized NLU trainer')
+	version = pkg_resources.require('projectalice-nlu-trainer')[0].version
+	print(f'Starting Project Alice offshore NLU trainer v. {version}')
 
 	trainer = NLUTrainer(hostname=host, port=port, user=user, password=password, tlsFile=tls_file)
 	try:
